@@ -6,7 +6,9 @@ use List::Util qw(first);
 use Carp qw(carp croak);
 use Params::Validate qw(:all);
 use File::Basename ();
+use Scalar::Util ();
 
+use Getopt::Long::Descriptive::Opts;
 use Getopt::Long::Descriptive::Usage;
 
 =head1 NAME
@@ -15,11 +17,11 @@ Getopt::Long::Descriptive - Getopt::Long with usage text
 
 =head1 VERSION
 
-Version 0.082
+Version 0.083
 
 =cut
 
-our $VERSION = '0.082';
+our $VERSION = '0.083';
 
 =head1 DESCRIPTION
 
@@ -356,6 +358,7 @@ sub _build_describe_options {
 
     my %return;
     $usage->die unless GetOptions(\%return, grep { length } @specs);
+    my @given_keys = keys %return;
 
     for my $opt (keys %return) {
       my $newopt = _munge($opt);
@@ -377,8 +380,9 @@ sub _build_describe_options {
       $return{$name} = $new;
     }
 
-    my $opt_obj = $class->_new_opt_obj({
+    my $opt_obj = Getopt::Long::Descriptive::Opts->___new_opt_obj({
       values => { %method_map, %return },
+      given  => { map {; $_ => 1 } @given_keys },
     });
 
     return($opt_obj, $usage);
@@ -509,46 +513,6 @@ sub _mk_only_one {
   die "unimplemented";
 }
 
-my $OPT_CLASS_COUNTER = 1;
-
-sub _class_for_opt {
-  my ($gld_class, $arg) = @_;
-
-  my $values = $arg->{values};
-  my @bad = grep { $_ !~ /^[a-z_]\w*$/ } keys %$values;
-  Carp::confess "perverse option names given: @bad" if @bad;
-
-  my $class = "$gld_class\::__OPT__::" . $OPT_CLASS_COUNTER++;
-
-  {
-    no strict 'refs';
-    ${"$class\::VERSION"} = $gld_class->VERSION;
-    for my $opt (keys %$values) {
-      *{"$class\::$opt"} = sub { $_[0]->{ $opt } };
-    }
-  }
-
-  return $class;
-}
-
-sub _new_opt_obj {
-  my ($gld_class, $arg) = @_;
-  
-  my $class = $gld_class->_class_for_opt($arg);
-
-  # This is stupid, but the traditional behavior was that if --foo was not
-  # given, there is no $opt->{foo}; it started to show up when we "needed" all
-  # the keys to generate a class, but was undef; this wasn't a problem, but
-  # broke tests of things that were relying on not-exists like tests of %$opt
-  # contents or MooseX::Getopt which wanted to use things as args for new --
-  # undef would not pass an Int TC.  Easier to just do this. -- rjbs,
-  # 2009-11-27
-  my $obj = bless { %{ $arg->{values} } } => $class;
-  delete $obj->{$_} for grep { ! defined $obj->{$_} } keys %$obj;
-
-  return $obj;
-}
-
 =head1 CUSTOMIZING
 
 Getopt::Long::Descriptive uses L<Sub::Exporter|Sub::Exporter> to build and
